{% extends 'core_main_app/_render/admin/theme/tools/box.html' %}
{% load timestamptags %}

{% block box_title %}Disabled versions{% endblock %}

{% block box_body %}
<div id="model_version">
	<table class="table table-bordered table-hover">
        <tr>
            <th style="width: 15%">Version #</th>
            <th style="width: 20%">Status</th>
            <th style="width: 20%">Upload date</th>
            <th>Actions</th>
        </tr>
	    {% for version in data.version_manager.versions.disabled reversed %}
        <tr>
            <td>{{ version.index }}</td>
            <td style='font-weight:bold;color:red'>Disabled</td>
            <td>
                {{ version.object|print_datetime_from_object_id }}
            </td>

            <td>
                <span class='icon legend retrieve' objectid='{{ version.object }}'>
                    <a class="btn btn-secondary" href="#"><i class="fa fa-undo"></i> Restore</a>
                </span>
            </td>
        </tr>
        {% empty %}
        <tr class="empty">
            <td colspan="4">No disabled versions for this template</td>
        </tr>
        {% endfor %}
   </table>
</div>
{% endblock %}
