<h3>{{data.title}}</h3>
<div class="message-container">
    <p>
        We've emailed you instructions for setting your password, if an account exists with the email you entered.
        You should receive them shortly.
    </p>
    <p>
        If you don't receive an email, please make sure you've entered the address you registered with,
        and check your spam folder.
    </p>
    <a class="btn btn-lg btn-secondary float-right" href="{% url 'core_main_app_login' %}">Login page</a>
</div>
