{% load staticfiles %}
{% load tz_detect %}

{% comment %}
    Root file to be used in any theme. Contains all necessary (and up-to-date) libraries.
{% endcomment %}
<!DOCTYPE html>
<html lang="en">
<head>

    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1"/>
    {% block meta %}{% endblock %}
    <title>{% block title %}{% endblock %}</title>

    {% block core_css %}
        <link rel="stylesheet" href="{% static 'core_main_app/libs/bootstrap/4.4.1/css/bootstrap.min.css' %}" />
        <link rel="stylesheet" href="{% static 'core_main_app/libs/font-awesome/5.13.0/css/all.min.css' %}" />
        <link rel="stylesheet" href="{% static 'core_main_app/libs/jquery-ui/1.12.1/jquery-ui.css' %}" />
        <link rel="stylesheet" href="{% static 'core_main_app/libs/jquery-ui/1.12.1/jquery-ui.structure.css' %}" />
        <link rel="stylesheet" href="{% static 'core_main_app/libs/jquery-ui/1.12.1/jquery-ui.theme.css' %}" />
        <link rel="stylesheet" href="{% static 'core_main_app/common/css/word-wrap.css' %}" />
        <link rel="stylesheet" href="{% static 'core_main_app/common/css/table.css' %}" />
        <link rel="stylesheet" href="{% static 'core_main_app/common/css/buttons.css' %}" />
        <link rel="stylesheet" href="{% static 'core_main_app/common/css/fields.css' %}" />

        {% if DISPLAY_NIST_HEADERS %}
        <link rel="stylesheet" href="https://pages.nist.gov/nist-header-footer/css/nist-combined.css">
        {% endif %}
    {% endblock %}
    {% block theme_css %}{% endblock %}
    {% block app_css %}{% endblock %}
</head>
<body>
    {% block body %}{% endblock %}
    {% block app_modals %}{% endblock %}

    {% block core_js %}
        <script src="{% static 'core_main_app/libs/jquery/3.1.1/jquery.js' %}"></script>
        <script src="{% static 'core_main_app/libs/jquery-ui/1.12.1/jquery-ui.js' %}"></script>
        <script src="{% static 'core_main_app/libs/popper-js/popper.min.js' %}"></script>
        <script src="{% static 'core_main_app/libs/bootstrap/4.4.1/js/bootstrap.min.js' %}"></script>
        <script src="{% static 'core_main_app/common/js/csrf.js' %}"></script>
        <script src="{% static 'core_main_app/common/js/leave_notice.js' %}"></script>
        {% if DISPLAY_NIST_HEADERS %}
        <script src="https://pages.nist.gov/nist-header-footer/js/nist-header-footer.js" type="text/javascript" defer="defer"></script>
        {% endif %}
        {% include 'core_main_app/_render/common/messages.html' %}
    {% endblock %}
    {% block theme_js %}{% endblock %}
    {% block app_js %}{% endblock %}

    <!--
    From the documentation:
        Add the detection template tag to your site, ideally in your base layout just before the </body> tag
    -->
    {% tz_detect %}
</body>
</html>