{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}assign-workspace-modal{% endblock %}
{% block modal_title %} Change workspace {% endblock %}

{% block modal_body %}
<div class="alert alert-danger" id="banner_assign_workspace_errors" style="display: none;">
    <h4><i class="fa fa-exclamation-circle fa-1x"></i> Error</h4>
    <div id="form_assign_workspace_errors">
    </div>
</div>

<input type="hidden" class="{{data.document}}-id"/>
Select a new workspace to assign the record:
<div id="assign-workspace-form">
</div>
{% endblock %}

{% block modal_footer %}
<a class="btn btn-secondary pull-left" data-dismiss="modal" href="#"><i class="fa fa-times"></i> Cancel</a>
<a id="assign-workspace-yes" class="btn btn-primary" href="#"><i class="fa fa-check"></i> Change</a>
{% endblock %}

