{% autoescape off %}
To initiate the password reset process for your {{ user.get_username }} account,
click the link below:

{{ protocol }}://{{ domain }}{% url 'password_reset_confirm' uidb64=uid token=token %}

If clicking the link above doesn't work, please copy and paste the URL in a new browser
window instead.

Sincerely,
The CDCS Team
{% endautoescape %}