<a href="#" class="logo">
  <span class="logo-mini"><b>A</b>DB</span>
  <span class="logo-lg"><b>Admin</b> Dashboard</span>
</a>

<nav class="navbar navbar-expand-lg navbar-static-top" role="navigation">

  <!-- Navbar Left Menu -->
    <ul class="nav navbar-nav">
      <li class="nav-item">
        <a href="#" class="fas fa-bars nav-link" data-toggle="offcanvas" role="button">
          <span class="sr-only">Toggle navigation</span>
        </a>
      </li>
    </ul>





  <!-- Navbar Right Menu -->
    <ul class="nav navbar-nav ml-auto navbar-custom-menu">
      <li class="nav-item">
        <a class="nav-link" href="{% url 'core_main_app_homepage' %}"><i class="fa fa-chevron-left"></i> Go to the website</a>
      </li>
      <li class="nav-item user user-menu">
        <a class="nav-link" href="#">
          <i class="fa fa-user"></i> {{ user }}
        </a>
      </li>
    </ul>
</nav>