
{% block modal_id %}edit-xslt-modal{% endblock %}
{% block modal_title %}Edit XSLT{% endblock %}

{% block modal_body %}
<div class="alert alert-danger" id="error-div-xslt" class="hidden">
    <a href="#" class="close" data-hide="alert" aria-label="close">&times;</a>
    <h4><i class="fa fa-exclamation-circle"></i> Error</h4>
    <div id="error-message"></div>
</div>
<form id="edit-xslt-form">
    <input id="edit-xslt-id" type="hidden" value=""/>
    <div class="form-group">
        <label for="edit-xslt-name">Enter new XSLT name</label>
        <input id="edit-xslt-name" class="form-control" type="text" value="" placeholder="new_xslt_name"/>
    </div>
</form>
{% endblock %}

{% block modal_footer %}
<a class="btn btn-secondary pull-left" data-dismiss="modal" href="#"><i class="fa fa-times"></i> Close</a>
<a id="edit-xslt-save" class="btn btn-primary" href="#"><i class="fa fa-save"></i> Save</a>
{% endblock %}