# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-09-26 14:40
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Main',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'verbose_name': 'core_main_app',
                'permissions': (('publish_data', 'Can publish data'),),
                'default_permissions': (),
            },
        ),
    ]
