{% extends "core_main_app/_render/admin/theme/tools/box.html" %}
{%load get_attribute%}

{% block box_title %}Error{% endblock %}

{% block box_body %}
    <div class="alert alert-danger">
        <i class="fa fa-warning"></i>
        {{ data | get_attribute:"error" }}
    </div>
{% endblock %}