{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}
{% block modal_id %}disable-version-modal{% endblock %}
{% block modal_title %}Disable Version <i class="disable-version-number"></i>{% endblock %}

{% block modal_body %}
<input id="disable-version-id" type="hidden" value=""/>
<p>
    The version won't be erased from this website but nobody will be able to switch to this version anymore.
</p>
<p>Are you sure you want to disable this version?</p>
{% endblock %}

{% block modal_footer %}
<button class="btn btn-secondary pull-left" data-dismiss="modal"><i class="fa fa-times"></i> No</button>
<button id="disable-version-yes" class="btn btn-danger"><i class="fa fa-check"></i> Yes</button>
{% endblock %}
