#form_start {
    overflow: auto;
}

.empty {
    text-align: center;
    font-style: italic;
    color: gray;
}

form>input {
    margin-bottom: 0.5em;
}

input[type=file] {
    border: solid 1px #f1f1f1;
    border-radius: 2px;
    display: block;
}