<div class="alert alert-error" id="banner_edit_errors" style="display: none;">
    <h4><i class="fa fa-exclamation-circle fa-1x"></i> Error</h4>
    <div id="form_edit_errors">
    </div>
</div>

<form id="assign-workspace-form" method="post">
    {% csrf_token %}
    {{ assign_workspace_form }}
    <div id="assign-workspace-errors" style="color:red;">
        {{ errors | safe}}
    </div>
    <br/>
</form>