{% load staticfiles %}

{% block section_title %} {{ data.object_name }}{% endblock %}

{% block section_content %}
<div class="row">
    <div class="col-md-12">
        {% include 'core_main_app/common/xslt/list/table.html' %}
    </div>
</div>
{% endblock %}