{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}disable-template-modal{% endblock %}
{% block modal_title %}Disable {{ data.object_name }} <i class="disable-template-name"></i>{% endblock %}

{% block modal_body %}
<input id="disable-template-id" type="hidden" value=""/>
<p>
    The {{ data.object_name }} won't be erased from this website but nobody will be able to create or edit document that have been
    created with it.
</p>
<p>Are you sure you want to disable this {{ data.object_name }}?</p>
{% endblock %}

{% block modal_footer %}
<button class="btn btn-secondary pull-left" data-dismiss="modal"><i class="fa fa-times"></i> No</button>
<button id="disable-template-yes" class="btn btn-danger"><i class="fa fa-check">

</i> Yes</button>
{% endblock %}
