<div id="{% block modal_id %}{% endblock %}" class="modal fade modal-default" tabindex="-1" role="dialog">
  <div class="modal-dialog modal-lg {% block modal_class %}{% endblock %}" role="document">
    <div class="modal-content">
      <div class="modal-header">
        <h5 class="modal-title">{% block modal_title %}{% endblock %}</h5>
        <button type="button" class="close" data-dismiss="modal" aria-label="Close">
          <span aria-hidden="true">&times;</span>
        </button>
      </div>
      <div class="modal-body">
        {% block modal_body %}{% endblock %}
      </div>
      <div class="modal-footer">
          {% block modal_footer %}{% endblock %}
      </div>
    </div>
  </div>
</div>