{% if data.validlink %}
<h3>{{data.title}}</h3>
<form method="post">
  {% csrf_token %}
  {{ data.form.as_p }}
  <button class="btn btn-lg btn-secondary float-right" type="submit">Change password</button>
</form>
{% else %}
<h3>{{data.title}}</h3>
<p>
  The password reset link was invalid, possibly because it has already been used.
  Please request a new password reset.
</p>
{% endif %}
